/*
 * Copyright  2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2007 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */

package com.sun.spot.dmamemory;

import java.util.Hashtable;


/**
 * Interface to a manager of uncached memory suitable for use as DMA buffers.
 */
public interface IDMAMemoryManager {

	/**
	 * Try to obtain memory for a DMA buffer.
	 * 
	 * @param size The size of the required buffer
	 * @param comment A string that describes the usage (so we can tell who's using what)
	 * @return The address of the allocated buffer
	 * @throws NotEnoughDMAMemoryException
	 */
	int getBuffer(int size, String comment) throws NotEnoughDMAMemoryException;

	/**
	 * Find out the size of the biggest available DMA memory buffer
	 * 
	 * @return The size of the biggest buffer that is available
	 */
	int getMaxAvailableBufferSize();

	/**
	 * Release a previously allocated buffer
	 * 
	 * @param memAddr The address of the buffer being released
	 */
	void releaseBuffer(int memAddr);

	/**
	 * Get information about current memory allocations. Each entry in the table
	 * represents an allocation. The key is the comment string provided when the
	 * buffer was allocated; the value is the size of the buffer.
	 * 
	 * @return The table of allocations
	 */
	Hashtable getAllocationDetails();

}
