/*
 * Copyright  2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2007 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */

package com.sun.spot.dmamemory.proxy;

import java.util.Hashtable;

import com.sun.spot.dmamemory.IDMAMemoryManager;
import com.sun.spot.dmamemory.NotEnoughDMAMemoryException;
import com.sun.spot.interisolate.NumberReplyEnvelope;
import com.sun.spot.interisolate.ReplyEnvelope;
import com.sun.spot.interisolate.RequestSender;
import com.sun.spot.peripheral.SpotFatalException;
import com.sun.squawk.io.mailboxes.NoSuchMailboxException;

/**
 * Acts as a proxy for DMAMemoryManager in child isolates
 */
public class ProxyDMAMemoryManager implements IDMAMemoryManager {

	public static final String DMA_MEMORY_SERVER = "DMA_MEMORY_SERVER";
	private RequestSender requestSender;

	/**
	 * For use by the SPOT library only - to get a handle to a DMA Memory Manager use
	 * Spot.getInstance().getDMAMemoryManager()
	 */
	public ProxyDMAMemoryManager() {
		try{
			requestSender = RequestSender.lookup(DMA_MEMORY_SERVER);						
		} catch (NoSuchMailboxException ex) {
			throw new RuntimeException(ex.getMessage());
		}
	}

	public Hashtable getAllocationDetails() {
		throw new SpotFatalException("getAllocationDetails is not implemented for child isolates");
	}

	public int getBuffer(int size, String comment) throws NotEnoughDMAMemoryException {
		ReplyEnvelope resultEnvelope = requestSender.send(new GetBufferCommand(size, comment));
		try {
			resultEnvelope.checkForThrowable();
		} catch (RuntimeException e) {
			throw e;
		} catch (NotEnoughDMAMemoryException e) {
			throw e;
		} catch (Throwable e) {
			throw new SpotFatalException("Unexpected exception: " + e);
		}
		return ((NumberReplyEnvelope) resultEnvelope).getIntContents();
	}

	public int getMaxAvailableBufferSize() {
		ReplyEnvelope resultEnvelope = requestSender.send(new GetMaxAvailableBufferSizeCommand());
		resultEnvelope.checkForRuntimeException();
		return ((NumberReplyEnvelope) resultEnvelope).getIntContents();
	}

	public void releaseBuffer(int memAddr) {
		ReplyEnvelope resultEnvelope = requestSender.send(new ReleaseBufferCommand(memAddr));
		resultEnvelope.checkForRuntimeException();
	}

}
