/*
 * Copyright  2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */

package com.sun.spot.flashmanagement;

import java.util.Date;
import java.util.Vector;

import com.sun.spot.peripheral.SpotFatalException;

class FlashFileDescriptor implements IExtendedFlashFileInfo {

	private static final short OBSOLETE_FLAG_MASK = 0x1;
	private int fileSize;
	private Vector sectors;
	private String name;
	private int virtualAddress;
	private long lastModifiedMillis;
	private String comment;
	private short flags;
	private int allocatedSpace;
	
	/**
	 * @return Returns the comment.
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * @param comment The comment to set.
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}

	/**
	 * @return Returns the lastModifiedMillis.
	 */
	public long lastModified() {
		return lastModifiedMillis;
	}

	/**
	 * @param lastModifiedMillis The lastModifiedMillis to set.
	 */
	public void setLastModifiedMillis(long lastModifiedMillis) {
		this.lastModifiedMillis = lastModifiedMillis;
	}

	/**
	 * @return Returns the virtualAddress.
	 */
	public int getVirtualAddress() {
		return virtualAddress;
	}

	/**
	 * @param virtualAddress The virtualAddress to set.
	 */
	public void setVirtualAddress(int virtualAddress) {
		this.virtualAddress = virtualAddress;
	}

	public FlashFileDescriptor(String name, int fileSize, Vector sectorsForFile, int virtualAddress, long lastModifiedMillis, String comment, short flags) {
		this.name = name;
		this.fileSize = fileSize;
		this.virtualAddress = virtualAddress;
		this.lastModifiedMillis = lastModifiedMillis;
		this.comment = comment;
		this.flags = flags;
		sectors = sectorsForFile;
		calculateAllocatedSpace();
	}

	/**
	 * @param currentSector
	 * @return
	 */
	public IAddressableNorFlashSector getNextSector(IAddressableNorFlashSector sector) {
		for (int i = 0; i < sectors.size(); i++) {
			IAddressableNorFlashSector s = (IAddressableNorFlashSector) sectors.elementAt(i);
			if (s == sector) {
				if (i == sectors.size() - 1) {
					throw new SpotFatalException("FlashFileDescriptor has been asked for a sector beyond the allocated space");
				}
				return (IAddressableNorFlashSector) sectors.elementAt(i+1);
			}
		}
		throw new SpotFatalException("FlashFileDescriptor: sector not found in getNextSector");
	}

	/**
	 * @return the size of the file
	 */
	public int length() {
		return fileSize;
	}

	/**
	 * @return the number of bytes allocated for this file
	 */
	public int getAllocatedSpace() {
		return allocatedSpace;
	}

	private void calculateAllocatedSpace() {
		allocatedSpace = 0;
		for (int i = 0; i < sectors.size(); i++) {
			allocatedSpace += ((IAddressableNorFlashSector)sectors.elementAt(i)).getSize();
		}
	}

	/**
	 * @return
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return
	 */
	public IAddressableNorFlashSector getFirstSector() {
		return (IAddressableNorFlashSector) sectors.firstElement();
	}

	/**
	 * @param numOfBytesInFile
	 */
	public void setFileSize(int numOfBytesInFile) {
		fileSize = numOfBytesInFile;
	}

	/**
	 * @return the sectors allocated to this file
	 */
	public IAddressableNorFlashSector[] getSectors() {
		IAddressableNorFlashSector[] addressableNorFlashSectors = new IAddressableNorFlashSector[sectors.size()];
		sectors.copyInto(addressableNorFlashSectors);
		return addressableNorFlashSectors;
	}

	public boolean isMapped() {
		if (!isAddressed()) {
			return false;
		}
		int offset = 0;
		for (int i = 0; i < sectors.size(); i++) {
			IAddressableNorFlashSector addressableNorFlashSector = (IAddressableNorFlashSector)sectors.elementAt(i);
			if (virtualAddress + offset != addressableNorFlashSector.getVirtualStartAddressAsInt()) {
				return false;
			}
			offset += addressableNorFlashSector.getSize();
		}
		return true;
	}

	public void setObsolete(boolean b) {
		if (b)
			flags |= OBSOLETE_FLAG_MASK;
		else
			flags &= ~OBSOLETE_FLAG_MASK;
	}

	public boolean isObsolete() {
		return (flags & OBSOLETE_FLAG_MASK) != 0;
	}

	short getFlags() {
		return flags;
	}

	public boolean isAddressed() {
		return virtualAddress != 0;
	}

	public void addSector(IAddressableNorFlashSector sector) {
		sectors.addElement(sector);
		calculateAllocatedSpace();
	}

	public void removeSector(IAddressableNorFlashSector sector) {
		if (!sectors.removeElement(sector)) {
			throw new SpotFatalException("Sector " + sector.getSectorNumber() + 
					" is not part of " + name);
		}
		calculateAllocatedSpace();
	}

	public boolean exists() {
		return true;
	}

	public int[] getSectorNumbers() {
		int[] result = new int[sectors.size()];
		for (int i = 0; i < result.length; i++) {
			result[i] = ((IAddressableNorFlashSector)sectors.elementAt(i)).getSectorNumber();
		}
		return result;
	}

	public String toString() {
		String lineSeparator = System.getProperty("line.separator");
		StringBuffer sb = new StringBuffer();
		sb.append(getName() + (getVirtualAddress() == 0 ? "  [unmapped]" : "  [mapped at 0x" + Integer.toHexString(getVirtualAddress()) + "]"));
		sb.append(lineSeparator);
		String string = "    sectors:";
		int[] sectors = getSectorNumbers();
		for (int j = 0; j < sectors.length; j++) {
			string += " 0x" + Integer.toHexString(sectors[j]);
		}
		sb.append(string);
		sb.append(lineSeparator);
		sb.append("    comment: " + getComment());
		sb.append(lineSeparator);
		sb.append("    last modified: " + new Date(lastModified()));
		sb.append(lineSeparator);
		sb.append("    size: " + length());
		sb.append(lineSeparator);
		return sb.toString();
	}
}
