/*
 * Copyright  2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */

package com.sun.spot.flashmanagement;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.sun.spot.peripheral.ConfigPage;
import com.sun.spot.peripheral.SpotFatalException;
import com.sun.squawk.VM;
import com.sun.squawk.peripheral.INorFlashSector;
import com.sun.squawk.peripheral.InsufficientFlashMemoryException;
import com.sun.squawk.vm.ChannelConstants;

/**
 * FlashManager
 *
 */
class FlashManager implements IFAT {

	public static final int FAT_IDENTIFIER = 0x12345678;

	Hashtable fileDescriptors;
	private Vector freeSectors;
	private IAddressableNorFlashSector fatSector;
	private INorFlashSectorFactory flashSectorFactory;
	
	FlashManager() {
	}

	public void initWithFreeSectors(int[] freeSectorNumbers, INorFlashSectorFactory factory) {
		flashSectorFactory = factory;
		
		fileDescriptors = new Hashtable();
		freeSectors = new Vector();
		
		for (int i = 0; i < freeSectorNumbers.length; i++) {
			freeSectors.addElement(factory.create(freeSectorNumbers[i], INorFlashSector.SYSTEM_PURPOSED));
		}
	}
	
	public void initFromStoredFAT(INorFlashSectorFactory factory) throws IOException {
		flashSectorFactory = factory;
		initFrom(new FlashFileInputStream(getFatSector()), factory);
	}

	void initFrom(InputStream is, INorFlashSectorFactory factory) throws IOException {
		flashSectorFactory = factory;

		fileDescriptors = new Hashtable();
		freeSectors = new Vector();

		DataInputStream dis = new DataInputStream(is);
		
		int identifierInt = dis.readInt();
		if (identifierInt != FAT_IDENTIFIER) {
			throw new SpotFatalException("This is not a FAT 0x" + Integer.toHexString(identifierInt));
		}

		int fileDescriptorCount = dis.readShort();

		for (int i = 0; i < fileDescriptorCount; i++) {
			short flags = dis.readShort();
			int virtualAddress = dis.readInt();
			Vector sectors = readSectorList(factory, dis);
			String name = dis.readUTF();
			int fileSize = dis.readInt();
			long lastModified = dis.readLong();
			String comment = dis.readUTF();
			FlashFileDescriptor flashFileDescriptor = new FlashFileDescriptor(name, fileSize, sectors, virtualAddress, lastModified, comment, flags);
			if (!flashFileDescriptor.isObsolete()) {
				if (virtualAddress != 0) {
					mapSectors(flashFileDescriptor);
				}
				fileDescriptors.put(name, flashFileDescriptor);
			} else {
				for (int j = 0; j < sectors.size(); j++) {
					freeSectors.addElement(sectors.elementAt(j));
				}
			}
		}
		Enumeration e = readSectorList(factory, dis).elements();
		while (e.hasMoreElements()) {
			freeSectors.addElement(e.nextElement());
		}
	}

	private Vector readSectorList(INorFlashSectorFactory factory, DataInputStream dis) throws IOException {
		Vector sectors = new Vector();
		int freeSectorCount = dis.readShort();
		for (int i = 0; i < freeSectorCount; i++) {
			sectors.addElement(factory.create(dis.readShort(), INorFlashSector.SYSTEM_PURPOSED));
		}
		return sectors;
	}

	public synchronized FlashFileDescriptor createFile(String name, int size) throws InsufficientFlashMemoryException {
		if (exists(name)) {
			throw new SpotFatalException("Cannot create file " + name + " - already exists");
		}
		Vector sectors = new Vector();
		int sizeSoFar = 0;
		int freeSectorIndex = 0;
		while (sizeSoFar < size) {
			if (freeSectorIndex < freeSectors.size()) {
				IAddressableNorFlashSector s = (IAddressableNorFlashSector) freeSectors.elementAt(freeSectorIndex++);
				sectors.addElement(s);
				sizeSoFar += s.getSize();
			} else {
				throw new InsufficientFlashMemoryException("Not enough space to create file " + name + " (wants " + size + ", " + sizeSoFar + " available)");
			}
		}
		Enumeration e = sectors.elements();
		while (e.hasMoreElements()) {
			freeSectors.removeElement(e.nextElement());
		}
		FlashFileDescriptor flashFileDescriptor = new FlashFileDescriptor(name, 0, sectors, 0, System.currentTimeMillis(), "", (short) 0);
		fileDescriptors.put(name, flashFileDescriptor);
		return flashFileDescriptor;
	}

	private int allocateVirtualAddress() {
		long guessedAddress = FlashFile.FIRST_FILE_VIRTUAL_ADDRESS;
		
		while (guessedAddress <= FlashFile.LAST_FILE_VIRTUAL_ADDRESS) {
			if (!isVirtualAddressInUse((int) guessedAddress)) {
				return (int) guessedAddress;
			}
			guessedAddress += FlashFile.VIRTUAL_ADDRESS_FILE_SPACING;
		}
		throw new SpotFatalException("Virtual address space exhausted");
	}

	private boolean isVirtualAddressInUse(int virtualAddress) {
		Enumeration e = fileDescriptors.elements();
		while (e.hasMoreElements()) {
			FlashFileDescriptor descriptor = (FlashFileDescriptor) e.nextElement();
			if (descriptor.getVirtualAddress() == virtualAddress && !descriptor.isObsolete()) {
				return true;
			}
		}
		return false;				
	}
	
	public boolean exists(String name) {
		return fileDescriptors.containsKey(name);
	}

	public FlashFileDescriptor getFileDescriptorFor(String name) throws FlashFileNotFoundException {
		if (exists(name)) {
			return (FlashFileDescriptor) fileDescriptors.get(name);
		}
		throw new FlashFileNotFoundException("File not found " + name);
	}

	public synchronized void deleteFile(String name) {
		if (exists(name)) {
			FlashFileDescriptor descriptor = (FlashFileDescriptor) fileDescriptors.remove(name);
			IAddressableNorFlashSector[] sectors = descriptor.getSectors();
			for (int i = 0; i < sectors.length; i++) {
				freeSectors.addElement(sectors[i]);
			}
		}
	}
	
	public synchronized void writeFAT() throws IOException {
		FlashFileOutputStream flashFileOutputStream = new FlashFileOutputStream(getFatSector());
		storeOn(flashFileOutputStream);
		flashFileOutputStream.close();
	}

	FlashFileDescriptor[] getFileDescriptors() {
		FlashFileDescriptor[] result = new FlashFileDescriptor[fileDescriptors.size()];
		Enumeration e = fileDescriptors.elements();
		int index = 0;
		while (e.hasMoreElements()) {
			result[index++] = (FlashFileDescriptor) e.nextElement();	
		}
		return result;
	}

	Vector getFreeSectors() {
		return freeSectors;
	}

	void storeOn(OutputStream os) throws IOException {
		DataOutputStream dos = new DataOutputStream(os);
		dos.writeInt(FAT_IDENTIFIER);
		dos.writeShort(fileDescriptors.size());
		Enumeration e = fileDescriptors.elements();
		while (e.hasMoreElements()) {
			FlashFileDescriptor descriptor = (FlashFileDescriptor) e.nextElement();
			dos.writeShort(descriptor.getFlags());
			dos.writeInt(descriptor.getVirtualAddress());
			IAddressableNorFlashSector[] sectors = descriptor.getSectors();
			dos.writeShort(sectors.length);
			for (int i = 0; i < sectors.length; i++) {
				dos.writeShort(sectors[i].getSectorNumber());
			}
			dos.writeUTF(descriptor.getName());
			dos.writeInt(descriptor.length());
			dos.writeLong(descriptor.lastModified());
			dos.writeUTF(descriptor.getComment());
		}
		dos.writeShort(freeSectors.size());
		e = freeSectors.elements();
		while (e.hasMoreElements()) {
			dos.writeShort(((IAddressableNorFlashSector) e.nextElement()).getSectorNumber());
		}
	}

	public synchronized void remap(String filename) {
		remap((FlashFileDescriptor) fileDescriptors.get(filename));
	}

	private void remap(FlashFileDescriptor flashFileDescriptor) {
		if (!flashFileDescriptor.isAddressed()) {
			flashFileDescriptor.setVirtualAddress(allocateVirtualAddress());
		}
		mapSectors(flashFileDescriptor);
		reprogramMMU();
	}

	private void mapSectors(FlashFileDescriptor descriptor) {
		IAddressableNorFlashSector[] sectors = descriptor.getSectors();
		int virtualAddress = descriptor.getVirtualAddress();
		for (int i = 0; i < sectors.length; i++) {
			sectors[i].setVirtualAddress(virtualAddress);
			virtualAddress = virtualAddress + sectors[i].getSize();
		}
	}

	void reprogramMMU() {
		VM.execSyncIO(ChannelConstants.REPROGRAM_MMU, 0, 0, 0, 0, 0, 0, null, null);
	}

	public void validateVirtualAddress(int virtualAddress) {
		long longVirtualAddress = virtualAddress & 0xFFFFFFFFL;
		if (longVirtualAddress < FlashFile.FIRST_FILE_VIRTUAL_ADDRESS)
			throw new IllegalArgumentException("Virtual address 0x" + Integer.toHexString(virtualAddress) + " is too low");
		if (longVirtualAddress > FlashFile.LAST_FILE_VIRTUAL_ADDRESS)
			throw new IllegalArgumentException("Virtual address 0x" + Integer.toHexString(virtualAddress) + " is too high");
		if (virtualAddress % FlashFile.VIRTUAL_ADDRESS_FILE_SPACING != 0)
			throw new IllegalArgumentException("Virtual address 0x" + Integer.toHexString(virtualAddress) + " is not on a valid boundary");
		if (isVirtualAddressInUse(virtualAddress))
			throw new IllegalArgumentException("Virtual address 0x" + Integer.toHexString(virtualAddress) + " is already allocated");
	}

	/**
	 * @return Returns the fatSector.
	 */
	private IAddressableNorFlashSector getFatSector() {
		if (fatSector == null) {
			fatSector = flashSectorFactory.create(ConfigPage.FAT_SECTOR_NUMBER, INorFlashSector.SYSTEM_PURPOSED);
		}
		return fatSector;
	}

	public synchronized boolean rename(FlashFileDescriptor fileDescriptor, String newName) throws IOException {
		if (exists(newName)) {
			return false;
		} else {
			fileDescriptors.remove(fileDescriptor.getName());
			fileDescriptor.setName(newName);
			fileDescriptors.put(newName, fileDescriptor);
			return true;
		}
	}

	/**
	 * @param fileDescriptor
	 * @return sector found.
	 * @throws InsufficientFlashMemoryException 
	 */
	public synchronized IAddressableNorFlashSector getExtraSector(FlashFileDescriptor fileDescriptor) throws InsufficientFlashMemoryException {
		if (freeSectors.isEmpty()) {
			throw new InsufficientFlashMemoryException("Not enough space to allocate sector for " + fileDescriptor.getName());
		}
		boolean mapped = fileDescriptor.isMapped();
		IAddressableNorFlashSector sector = (IAddressableNorFlashSector) freeSectors.elementAt(0);
		freeSectors.removeElementAt(0);
		fileDescriptor.addSector(sector);
		if (mapped) {
			remap(fileDescriptor);
		}
		return sector;
	}

	/**
	 * @param fileDescriptor
	 * @param sector
	 */
	public synchronized void releaseSector(FlashFileDescriptor fileDescriptor, IAddressableNorFlashSector sector) {
		boolean mapped = fileDescriptor.isMapped();
		fileDescriptor.removeSector(sector);
		freeSectors.addElement(sector);
		if (mapped) {
			remap(fileDescriptor);
		}
	}

	public IExtendedFlashFileInfo[] getFileInfos() {
		return getFileDescriptors();
	}

	public synchronized int[] getFreeSectorIndices() {
		int[] result = new int[freeSectors.size()];
		for (int i = 0; i < result.length; i++) {
			result[i] = ((IAddressableNorFlashSector)freeSectors.elementAt(i)).getSectorNumber();
		}
		return result;
	}
	
	public String toString() {
		String lineSeparator = System.getProperty("line.separator");
		IExtendedFlashFileInfo[] fileInfos = getFileInfos();
		StringBuffer sb = new StringBuffer(100);
		sb.append("FAT contains " + fileInfos.length + " files.");
		sb.append(lineSeparator);
		for (int i = 0; i < fileInfos.length; i++) {
			sb.append(" " + i + ": " + fileInfos[i].toString());
		}
		String string = " Free sectors: ";
		int[] freeSectorIndices = getFreeSectorIndices();
		for (int i = 0; i < freeSectorIndices.length; i++) {
			string += " 0x" + Integer.toHexString(freeSectorIndices[i]);
		}
		sb.append(string);
		sb.append(lineSeparator);
		return sb.toString();
	}
}

