/*
 * Copyright  2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2007 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */

package com.sun.spot.flashmanagement;

import java.io.IOException;

/**
 * IFlashFileInfo provides a read-only interface to files in the flash filing system
 */
public interface IFlashFileInfo {

	/**
	 * Get the length of the file with this name
	 * @return the length in bytes of the valid portion of the file with this name
	 * @throws IOException
	 */
	public abstract int length() throws IOException;

	/**
	 * Get the virtual address of a mapped file
	 * @return the virtual address of the file, or 0 if it is not a mapped file
	 * @throws IOException
	 */
	public abstract int getVirtualAddress() throws IOException;

	/**
	 * Get the comment for this file
	 * @return the comment attached to the file's descriptor
	 * @throws IOException
	 */
	public abstract String getComment() throws IOException;

	/**
	 * Get the time at which this file was last modified
	 * @return the time (as returned by System.currentTimeMillis()) at which this file was last modified.
	 * @throws IOException
	 */
	public abstract long lastModified() throws IOException;

	/**
	 * Check whether the file with this name is obsolete
	 * @return true if this file is marked obsolete, false if it is not
	 * @throws IOException
	 */
	public abstract boolean isObsolete() throws IOException;

	/**
	 * Get the name of this file
	 * @return the name of this file
	 */
	public abstract String getName();
}
