/*
 * Copyright  2006 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
package com.sun.spot.io.j2me.remoteprinting;

import com.sun.spot.peripheral.Spot;
import com.sun.spot.peripheral.ota.ISpotAdminConstants;
import com.sun.spot.util.Utils;
import com.sun.squawk.Isolate;

/**
 * Manages the creation and removal of remote print connections
 */
public class RemotePrintManager implements IRemotePrintManager {

	private static IRemotePrintManager remotePrintManager;

	public synchronized static IRemotePrintManager getInstance() {
		if (remotePrintManager == null) {
			remotePrintManager = new RemotePrintManager();
		}
		return remotePrintManager;
	}

	private String divertingTo;
	private boolean diverted;

	RemotePrintManager() {}
	
	public synchronized void redirectOutputStreams(String basestationAddr) {
		if (Utils.isOptionSelected("spot.remote.print.disabled", false)) {
			return;
		}
		Isolate isolate = Isolate.currentIsolate();
		if (diverted && !divertingTo.equals(basestationAddr)) {
			diverted = false;
			isolate.removeOut("remoteprint://"+divertingTo+":" + getEchoPort());
			isolate.removeErr("remoteprint://"+divertingTo+":" + getEchoPort());			
		}
		if (!diverted ) {
			divertingTo = basestationAddr;
//			isolate.removeOut("serial:");
//			isolate.removeErr("serial:");
			isolate.addOut("remoteprint://"+basestationAddr+":" + getEchoPort());
			isolate.addErr("remoteprint://"+basestationAddr+":" + getEchoPort());
			diverted = true;
		}
	}

	public synchronized void cancelRedirect() {
		Isolate isolate = Isolate.currentIsolate();
		if (diverted) {
			diverted = false;
			isolate.removeOut("remoteprint://"+divertingTo+":" + getEchoPort());
			isolate.removeErr("remoteprint://"+divertingTo+":" + getEchoPort());			
		}
	}

	private int getEchoPort() {
		return Spot.getInstance().isMasterIsolate() ?
				ISpotAdminConstants.MASTER_ISOLATE_ECHO_PORT :
				ISpotAdminConstants.CHILD_ISOLATE_ECHO_PORT;
	}

	public void noteRedirection(String remoteAddress) {
		if(!diverted) {
			diverted = true;
			divertingTo = remoteAddress;
		}
	}
}
