/*
 * Copyright 2005 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 */
package com.sun.spot.io.j2me.socket;

import java.io.IOException;
import java.io.InputStream;

/**
 * <p>
 * Socket specific InputSteam. This class handles the passing of protocol information while remaining seamless to the user.
 * </p>
 * @author Martin Morissette
 *
 */
public class SocketProtocolInputStream extends InputStream implements SpotSocketProtocol {

    private InputStream in;

    /**
     * Create the input stream from another InputStream.
     * @param in InputStream to be used
     */
    public SocketProtocolInputStream(InputStream in) {
        this.in = in;
    }

    /**
     * Read data from the stream. 
     */
    public int read() throws IOException {
        int data = in.read();

        if (data == ESCAPE_CHAR) {
            data = in.read();
            switch (data) {
                case CONNECTION_CLOSE:
                    data = -1;
                    break;
                case IOEXCEPTION:
                    throw new IOException(readProtocolString());
                case ESCAPE_CHAR:
                    break;
                default:
                    throw new IOException("Unexpected escape character in protocol: " + data);
            }
        }
        return data;
    }

    /**
     * <p>
     * Reads a string within the socket protocol. Protocol strings are used to send extra information.
     * For example, remote exceptions
     * </p>
     * @return String transfered within the socket protocol (invisible to the user)
     * @throws IOException
     */
    private String readProtocolString() throws IOException {
        int data;
        int size = 0;
        int shiftBy = 0;
        
        while (true) {
            data = in.read();
            size = ((data & 0x7F) << shiftBy) | size;
            shiftBy += 7;
            if ((data & 0x80) == 0) {
                break;
            }
        }
        
        byte[] bytes = new byte[size];
        int read = in.read(bytes, 0, size);
        return new String(bytes, 0, read);
    }
    
    
    /**
     * Close the InputStream.
     */
    public void close() throws IOException{
        in.close();
        super.close();
    }

}
