/*
 * Copyright  2006 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
package com.sun.spot.peripheral.basestation;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import com.sun.spot.peripheral.radio.I802_15_4_MAC;

public abstract class MACCommand implements ICommand {
	private static final int SUCCESS = 1;
	private static final int FAILURE = 0;

	public static final byte GetNullPacketAfterAckWaitCommand = 0;
	public static final byte GetChannelAccessFailureCommand = 1;
	public static final byte GetNoAckCommand = 2;
	public static final byte GetPLMETransmitCommand = 3;
	public static final byte GetWrongAckCommand = 4;
	public static final byte MCPSDataIndicationCommand = 5;
	public static final byte MCPSDataRequestCommand = 6;
	public static final byte MLMEGetCommand = 7;
	public static final byte MLMEResetCommand = 8;
	public static final byte MLMERxEnableCommand = 9;
	public static final byte MLMESetCommand = 10;
	public static final byte MLMEStartCommand = 11;
	public static final byte SetPLMETransmitPowerCommand = 12;
	
	public static final byte ResetProxyCommand = 101;
	

	private int uid;	

	public final void writeOnto(DataOutputStream dataOutputStream) throws IOException {
		dataOutputStream.writeByte(classIndicatorByte());
		writeParametersOnto(dataOutputStream);
	}

	protected abstract byte classIndicatorByte();

	protected void writeParametersOnto(DataOutputStream dataOutputStream) throws IOException {
		//default for no-arg commands
	}

	public void populateFrom(byte[] inputBuffer, int startingOffset) throws IOException {
		//default for no-arg commands
	}

	public int writeResultOnto(byte[] outputBuffer, int startingOffset, I802_15_4_MAC mac) throws IOException {
		try {
			prepareResultOrExecute(mac);
			outputBuffer[startingOffset] = SUCCESS;
			return writePreparedResult(outputBuffer, startingOffset+1) + 1; // 1 for SUCCESS byte
		} catch (Throwable ex) {
			ex.printStackTrace();
			outputBuffer[startingOffset] = FAILURE;
			int exceptionLength = writeExceptionDetails(outputBuffer, startingOffset, ex);
			return exceptionLength + 1; // 1 for FAILURE byte
		}
	}

	private int writeExceptionDetails(byte[] outputBuffer, int startingOffset, Throwable ex) throws IOException {
		ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		DataOutputStream dataOutputStream = new DataOutputStream(byteArrayOutputStream);
		
		dataOutputStream.writeUTF(ex.getClass().getName());
		dataOutputStream.writeUTF(ex.getMessage() == null ? "" : ex.getMessage());
		
		byte[] exceptionMsgContent = byteArrayOutputStream.toByteArray();
		System.arraycopy(exceptionMsgContent, 0, outputBuffer, startingOffset+1, exceptionMsgContent.length);
		return exceptionMsgContent.length;
	}

	protected abstract void prepareResultOrExecute(I802_15_4_MAC mac) throws Throwable;
	
	protected int writePreparedResult(byte[] outputBuffer, int startingOffset) throws IOException {
		//default for void commands
		return 0;
	}

	public final Object resultFrom(DataInputStream dataInputStream, ICreateExceptions exceptionCreator) throws Throwable {
		boolean isOk = dataInputStream.readBoolean();
		if (isOk) {
			return readResultFrom(dataInputStream);
		} else {
			String exceptionClassName = dataInputStream.readUTF();
			String exceptionMessage = dataInputStream.readUTF();
			return exceptionCreator.throwException(exceptionClassName, exceptionMessage);
		}
	}

	protected Object readResultFrom(DataInputStream dataInputStream) throws IOException {
		//default for void commands
		return null;
	}
	
	public int getUID() {
		return this.uid;
	}
	
	public long getTimeout() {
		// by default, wait for ever
		return 0;
	}

	public void setUID(int uid) {
		this.uid = uid;
	}
}
