/*
 * Copyright  2006 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
package com.sun.spot.peripheral.basestation;

import java.io.DataInputStream;
import java.io.IOException;

import com.sun.spot.peripheral.radio.I802_15_4_MAC;
import com.sun.spot.peripheral.radio.RadioPacket;
import com.sun.spot.util.Queue;


public class MCPSDataIndicationCommand extends MACCommand {

	protected static final int INCOMING_QUEUE_HIGH_WATER = 1000;
	protected static final int INCOMING_QUEUE_LOW_WATER  = 900;
	private Queue incomingPackets;
	private RadioPacket radioPacket;
	protected boolean stopped = false;

	public MCPSDataIndicationCommand with(final I802_15_4_MAC mac) {
		incomingPackets = new Queue();
		new Thread() {
			public void run() {
				while (true) {
					RadioPacket radioPacket = RadioPacket.getDataPacket();
					mac.mcpsDataIndication(radioPacket);
					incomingPackets.put(radioPacket);
					int size = incomingPackets.size();
					if (size > INCOMING_QUEUE_HIGH_WATER) {
						System.err.println("[reached HIGH_WATER]");
						mac.mlmeSet(I802_15_4_MAC.MAC_RX_ON_WHEN_IDLE, I802_15_4_MAC.FALSE);
						mac.mlmeRxEnable(0);
						stopped = true;
					}
				}
			};
		}.start();
		return this;
	}

	protected void prepareResultOrExecute(I802_15_4_MAC mac) throws InterruptedException {
		radioPacket = (RadioPacket) incomingPackets.get();
		int size = incomingPackets.size();
		if (stopped && size < INCOMING_QUEUE_LOW_WATER) {
			System.err.println("[reached LOW_WATER]");
			mac.mlmeSet(I802_15_4_MAC.MAC_RX_ON_WHEN_IDLE, I802_15_4_MAC.TRUE);
			stopped = false;
		}			
	}
	
	protected int writePreparedResult(byte[] outputBuffer, int startingOffset) throws IOException {
		if (radioPacket == null) {
			System.err.println("[MCPSDataIndicationCommand] null packet");
			return 0; // should be resetting if packet is null
		} else {
			return radioPacket.writeWithoutTimestampOnto(outputBuffer, startingOffset);
		}
	}
	
	protected Object readResultFrom(DataInputStream dataInputStream) throws IOException {
		RadioPacket packet = RadioPacket.getDataPacket();
		packet.readWithoutTimestampFrom(dataInputStream);
		return packet;
	}
	
	protected byte classIndicatorByte() {
		return MCPSDataIndicationCommand;
	}

	public void reset() {
		Queue oldQueue = incomingPackets;
		incomingPackets = new Queue();
		oldQueue.stop(); // free up any waiting MACProxyWorkerThreads
	}
}
