/*
 * Copyright  2006 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
package com.sun.spot.util;

import java.io.*;
import javax.microedition.io.*;

import com.sun.squawk.VM;

/**
 * Simple class to listen to the serial input over the USB connection and
 * pass control to the bootloader. Means you do not have to push the reset
 * button on the SPOT when downloading new code.
 * 
 * To use (simplest way):
 * 
 * new BootloaderListener().start();
 * 
 * the BootloaderListener will exit the VM when a bootloader command is detected.
 * 
 * If you want to take your own action:
 * 
 * new BootloaderListener(myCallbackObject).start();
 * 
 * where myCallbackObject implements {@link com.sun.spot.util.IBootloaderListenerCallback IBootloaderListenerCallback}. 
 *
 * You can cancel the bootloader listener by calling cancel()
 * 
 * @author Ron Goldman / Syntropy
 */
 public class BootloaderListener extends Thread {   // Used to monitor the USB serial line
        
    private InputStream in;
    private boolean runBootLoaderListener = true;
    private IBootloaderListenerCallback callback = null;

    public BootloaderListener() {}
    
    public BootloaderListener(IBootloaderListenerCallback callback) {
    	this.callback = callback;
    }

    /**
     * Cleanup after ourself and stop running.
     */
    public void cancel() {
    	runBootLoaderListener = false;
        try {
            in.close();
        } catch (IOException ex) {
            // ignore any exceptions
        }
    }

    /**
     * Loop reading characters sent over USB connection and dispatch to bootloader when requested.
     */
    public void run () {
        try {
            in = Connector.openInputStream("serial://");

            while (runBootLoaderListener) {
                char c = (char)in.read();
                if ('A' <= c && c <= 'P') {
                	if (callback != null) {
                		callback.prepareToExit();
                	}
                	System.out.println("Exiting - detected bootloader command");
                    VM.stopVM(0);         // return control to bootloader
                }
            }
        } catch (IOException ex) {
            System.err.println("Exception while listening to serial line: " + ex);
        }
    }
}        
