/*
 * Copyright  2006 Sun Microsystems, Inc. All rights reserved. 
 * 
 * Sun Microsystems, Inc. has intellectual property rights relating to technology embodied in the product that is
 * described in this document. In particular, and without limitation, these intellectual property rights may
 * include one or more of the U.S. patents listed at http://www.sun.com/patents and one or more additional patents
 * or pending patent applications in the U.S. and in other countries.
 * 
 * U.S. Government Rights - Commercial software. Government users are subject to the Sun Microsystems, Inc.
 * standard license agreement and applicable provisions of the FAR and its supplements.
 * 
 * Use is subject to license terms. 
 * 
 * This distribution may include materials developed by third parties.Sun, Sun Microsystems, the Sun logo and
 * Java are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. and other countries. 
 * 
 * Copyright  2006 Sun Microsystems, Inc. Tous droits rservs.
 * 
 * Sun Microsystems, Inc. dtient les droits de proprit intellectuels relatifs  la technologie incorpore dans
 * le produit qui est dcrit dans ce document. En particulier, et ce sans limitation, ces droits de proprit
 * intellectuelle peuvent inclure un ou plus des brevets amricains lists  l'adresse http://www.sun.com/patents
 * et un ou les brevets supplmentaires ou les applications de brevet en attente aux Etats - Unis et dans les
 * autres pays.
 * 
 * L'utilisation est soumise aux termes du contrat de licence.
 * 
 * Cette distribution peut comprendre des composants dvelopps par des tierces parties.
 * Sun, Sun Microsystems, le logo Sun et Java sont des marques de fabrique ou des marques dposes de Sun
 * Microsystems, Inc. aux Etats-Unis et dans d'autres pays.
 */
package com.sun.spot.util;

import java.io.IOException;
import java.io.InputStream;

/**
 * Standalone class to provide crc calculations.
 */
public class CRC {
	/* Table of CRCs of all 8-bit messages. */
	private static int[] crc_table = make_crc_table();
	
	/**
	 * Return the CRC of the specified part of a byte array.
	 * 
	 * @param buf source of bytes
	 * @param offset offset in buf to first byte
	 * @param len number of bytes to include in calculation
	 * @return 16-bit CRC value
	 */
	public static short crc(byte[] buf, int offset, int len) {
		/* Update a running CRC with the bytes buf[0..len-1]--the CRC
		 * should be initialized to all 1's, and the transmitted value
		 * is the 1's complement of the final running CRC (see the
		 * crc() routine below)). */
		int c = 0xffff;

		for (int n = offset; n < len + offset; n++) {
			c = update_crc(buf[n], c);
		}

		return (short)(c ^ 0xffff);
	}
	
	/**
	 * Return the CRC value of the specified number of bytes from an InputStream.
	 * 
	 * @param stream to read bytes from
	 * @param flashedByteCount number of bytes to read
	 * @return 16-bit CRC value
	 * @throws IOException
	 */
	public static short crc(InputStream stream, int flashedByteCount) throws IOException {
		/* Update a running CRC with the bytes buf[0..len-1]--the CRC
		 * should be initialized to all 1's, and the transmitted value
		 * is the 1's complement of the final running CRC (see the
		 * crc() routine below)). */
		int c = 0xffff;

		for (int n = 0; n < flashedByteCount; n++) {
			c = update_crc((byte)stream.read(), c);
		}

		return (short)(c ^ 0xffff);
	}   

	private static int update_crc(byte b, int c) {
		return crc_table[(c ^ b) & 0xff] ^ (c >> 8);
	}
	
	/* Make the table for a fast CRC. */
	private static int[] make_crc_table() {
		int[] crc_table = new int[256];
		int c;
   
		for (int n = 0; n < 256; n++) {
			c = n;
			for (int k = 0; k < 8; k++) {
				if ((c & 1) != 0)
					c = 0x8408 ^ (c >> 1);
				else
					c = c >> 1;
			}
			crc_table[n] = c;
		}
		return crc_table;
	}

	
}
